
# Cache ... should clean this up ... packaging is not working
import pathlib
import os
import json
from successor.conventions import skater_model_suffix, keras_optimizer_from_name
from tensorflow import keras
from successor.skaters.scalarskaters.remote import SKLEARNED_CHAMPIONS, get_remote_info, get_remote_model_spec, SKL_EXAMPLE_REMOTE_PATH
from pprint import pprint


ROOT_PATH = str(pathlib.Path(os.path.dirname(os.path.realpath(__file__))).parent.parent.parent)
MODEL_CACHE = ROOT_PATH + os.path.sep + 'sklearnedmodelcache'
INFO_CACHE = ROOT_PATH + os.path.sep + 'sklearnedinfocache'


def local_info_path(skater_name, k: int, n_input: int):
    return INFO_CACHE + os.path.sep + skater_model_suffix(skater_name=skater_name, k=k, n_input=n_input)


def local_model_path(skater_name, k: int, n_input: int):
    return MODEL_CACHE + os.path.sep + skater_model_suffix(skater_name=skater_name,k=k,n_input=n_input)


def get_local_info(skater_name:str, k: int, n_input: int):
    with open(local_info_path(skater_name=skater_name, k=k, n_input=n_input),'rt') as fp:
        info = json.load(fp)
    return info





def save_model_locally(model: dict, skater_name: str, k: int, n_input: int):
    _file = local_model_path(skater_name=skater_name, k=k, n_input=n_input)
    with open(_file, 'wt') as fp:
        json.dump(obj=model,fp=fp)



def save_info_locally(info: dict, skater_name: str, k: int, n_input: int):
    _file = local_info_path(skater_name=skater_name, k=k, n_input=n_input)
    with open(_file, 'wt') as fp:
        json.dump(obj=info,fp=fp)



def copy_inventory_to_cache():
    for champ in SKLEARNED_CHAMPIONS:
        model = get_remote_model_spec(**champ)
        info  = get_remote_info(**champ)
        if model is not None and info is not None:
            save_model_locally(model=model,**champ)
            save_info_locally(info=info, **champ)



def get_local_model(skater_name:str, k: int, n_input: int):
    with open(local_model_path(skater_name=skater_name, k=k, n_input=n_input),'rt') as fp:
        model_json = fp.read()
    model = keras.models.model_from_json(model_json)
    return model

def get_local_compiled_model(skater_name:str,k:int,n_input:int):
    model = get_local_model(skater_name=skater_name,k=k,n_input=n_input)
    model_info = get_local_info(skater_name=skater_name,k=k,n_input=n_input)
    keras_optimizer = keras_optimizer_from_name(opt_name=model_info['keras_optimizer'], learning_rate=model_info['learning_rate'])
    model.compile(optimizer=keras_optimizer)
    return model



if __name__ == '__main__':
    print('example model location is ' + SKL_EXAMPLE_REMOTE_PATH)
    copy_inventory_to_cache()
    n_input = 160
    model = get_local_compiled_model(skater_name='tsa_aggressive_d0_ensemble',k=1,n_input=n_input)
    pprint(model)
    import numpy as np
    x = np.random.randn(300,1,n_input)
    y = model.predict(x)
    pass




def reflect(y:[float], n_input:int):
    """ Prepend to start of vector """
    if len(y)>=n_input:
        return y
    else:
        assert len(y),'Cannot extend zero length vector'
        y_extended = [c for c in y]
        while len(y_extended)<n_input:
            y_extended = list(reversed(y_extended)) + y_extended
        return y_extended[-n_input:]








import numpy as np


def nan_helper(y):
    # Lazy C&P https://stackoverflow.com/questions/6518811/interpolate-nan-values-in-a-numpy-array
    return np.isnan(y), lambda z: z.nonzero()[0]


def linear_interpolator(y):
    if len(y)==1:
        return y
    else:
        return interp_nans(y)


def interp_nans(x:[float],left=None, right=None, period=None)->[float]:
    """ [1 1 1 nan nan 2 2 nan 0] -> [1 1 1 1.3 1.6 2 2  1  0]
        Same conventions as https://numpy.org/doc/stable/reference/generated/numpy.interp.html
    """
    xp = [i for i, yi in enumerate(x) if np.isfinite(yi)]
    fp = [yi for i, yi in enumerate(x) if np.isfinite(yi)]
    return list(np.interp(x=list(range(len(x))), xp=xp, fp=fp,left=left,right=right,period=period))


import pathlib
import os
from getjson import getjson
from pprint import pprint
import json
from successor.skaters.scalarskaters.sklearnedinventory import SKLEARNED_CHAMPION_URLS
from tensorflow import keras
from successor.conventions import keras_optimizer_from_name, skater_model_suffix



SKL_REMOTE_PATH = 'https://raw.githubusercontent.com/microprediction/sklearned/main'
SKL_REMOTE_MODEL_PATH = SKL_REMOTE_PATH + '/champion_models'
SKL_REMOTE_INFO_PATH = SKL_REMOTE_PATH + '/champion_info'
SKL_EXAMPLE_REMOTE_PATH = SKL_REMOTE_MODEL_PATH + os.path.sep + 'tsa_aggressive_combined_ensemble_1_80.json'


def champion_from_url(url):
    # e.g. https: // raw.githubusercontent.com / microprediction / sklearned / main / champion_models / tsa_aggressive_combined_ensemble_1_80.json
    last_bit = url.split('/')[-1].replace('.json','')
    n_input = int(last_bit.split('_')[-1])
    k = int(last_bit.split('_')[-2])
    skater_name = '_'.join( last_bit.split('_')[:-2])
    return dict(skater_name=skater_name,k=k,n_input=n_input)

SKLEARNED_CHAMPIONS = [champion_from_url(url) for url in SKLEARNED_CHAMPION_URLS]


def remote_info_path(skater_name, k: int, n_input: int):
    return SKL_REMOTE_INFO_PATH + '/' + skater_model_suffix(skater_name=skater_name,k=k,n_input=n_input)


def remote_model_path(skater_name, k: int, n_input: int):
    return SKL_REMOTE_MODEL_PATH + '/' + skater_model_suffix(skater_name=skater_name, k=k, n_input=n_input)


def get_remote_model_spec(skater_name:str, k: int, n_input: int):
    _path = remote_model_path(skater_name=skater_name, k=k, n_input=n_input)
    _res = getjson(_path)
    if isinstance(_res,str):
        return json.loads(_res)
    elif isinstance(_res,dict):
        return _res
    else:
        print('Missing '+remote_model_path(skater_name=skater_name,k=k,n_input=n_input))


def get_remote_info(skater_name:str, k: int, n_input: int):
    _path = remote_info_path(skater_name=skater_name, k=k, n_input=n_input)
    _res = getjson(_path)
    if isinstance(_res,str):
        return json.loads(_res)
    elif isinstance(_res,dict):
        return _res
    else:
        print('Missing '+remote_info_path(skater_name=skater_name,k=k,n_input=n_input))


def get_remote_model(skater_name:str, k: int, n_input: int):
    _path = remote_model_path(skater_name=skater_name, k=k, n_input=n_input)
    _res = getjson(_path)
    if isinstance(_res,dict):
        _res = json.dumps(_res)
    model = keras.models.model_from_json(_res)
    return model


def get_remote_compiled_model(skater_name:str,k:int,n_input:int):
    model = get_remote_model(skater_name=skater_name, k=k, n_input=n_input)
    model_info = get_remote_info(skater_name=skater_name, k=k, n_input=n_input)
    keras_optimizer = keras_optimizer_from_name(opt_name=model_info['keras_optimizer'], learning_rate=model_info['learning_rate'])
    model.compile(optimizer=keras_optimizer)
    return model


